/*
 *   kgoodstuff - a button bar a la FvwmButtons for KDE
 *   Copyright (C) 1998  Juergen Vigna
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "kgoodstuff.h"
#include <kapp.h>
#include <kwmmapp.h>
#include "kgoodstuff_version.h"
#include <X11/keysym.h>

KGoodStuff *mainKGS;

class MyApp:public KWMModuleApplication {
public:
    MyApp( int &argc, char **argv, const QString& rAppName );
    virtual bool x11EventFilter( XEvent * );
};

MyApp::MyApp(int &argc, char **argv , const QString& rAppName):
        KWMModuleApplication(argc, argv, rAppName)
{
}

bool MyApp::x11EventFilter( XEvent * ev)
{
    if (ev->xany.window != None && 
        mainKGS->parentOfSwallowed(ev->xany.window)) {
        if (ev->type == ButtonPressMask) {
            XAllowEvents(qt_xdisplay(), SyncPointer, CurrentTime);
            mainKGS->parentOfSwallowed(ev->xany.window)->grabMouse();
        }
        ev->xany.window=mainKGS->parentOfSwallowed(ev->xany.window)->winId();
    }
#if 0
    if (ev->type == KeyPress){
        XKeyEvent* e = &ev->xkey;
        int kc = XKeycodeToKeysym(qt_xdisplay(), e->keycode, 0);
        int km = e->state & (ControlMask | Mod1Mask | ShiftMask);
        if( (kc == XK_F1)  && (km == Mod1Mask) ){
            mainKGS->showSystem();
            XAllowEvents(qt_xdisplay(), AsyncKeyboard, CurrentTime);
            return true;
        }
    }
#endif
    return KWMModuleApplication::x11EventFilter(ev);
}

#if 0
static void grabKey(KeySym keysym, unsigned int mod)
{
    static int NumLockMask = 0;
    
    if (!XKeysymToKeycode(qt_xdisplay(), keysym)) return; 
    if (!NumLockMask){
        XModifierKeymap* xmk = XGetModifierMapping(qt_xdisplay());
        int i;
    for (i=0; i<8; i++){
        if (xmk->modifiermap[xmk->max_keypermod * i] == 
            XKeysymToKeycode(qt_xdisplay(), XK_Num_Lock))
            NumLockMask = (1<<i); 
    }
    }
    XGrabKey(qt_xdisplay(),
             XKeysymToKeycode(qt_xdisplay(), keysym), mod,
             qt_xrootwin(), True,
             GrabModeAsync, GrabModeSync);
    XGrabKey(qt_xdisplay(),
             XKeysymToKeycode(qt_xdisplay(), keysym), mod | LockMask,
             qt_xrootwin(), True,
             GrabModeAsync, GrabModeSync);
    XGrabKey(qt_xdisplay(),
             XKeysymToKeycode(qt_xdisplay(), keysym), mod | NumLockMask,
             qt_xrootwin(), True,
             GrabModeAsync, GrabModeSync);
    XGrabKey(qt_xdisplay(),
             XKeysymToKeycode(qt_xdisplay(), keysym),
             mod | LockMask | NumLockMask,
             qt_xrootwin(), True,
             GrabModeAsync, GrabModeSync);
}
#endif

int main( int argc, char **argv )
{
    int v;
    const char *num = 0;

//    KApplication a( argc, argv );
    MyApp myapp(argc,argv,"kgoodstuff");

    for (v=1; v<argc; v++){
        if (QString("-version") == argv[v]) {
            printf(KGOODSTUFF_VERSION);
            printf("\n");
            printf("Copyright (C) 1998 Jrgen Vigna (jug@sad.it)\n\n");
            exit(0);
        } else if ((QString("-num") == argv[v]) && ((v+1) < argc)) {
            v++;
            num = argv[v];
        } else {
            printf(KGOODSTUFF_VERSION);
            printf("\n");
            printf("Copyright (C) 1998 Jrgen Vigna (jug@sad.it)\n\n");
            printf(i18n("Desktop Button Bar for the KDE Desktop Environment\n"));
            printf(i18n("Permitted arguments:\n"));
            printf(i18n("-help : displays this message\n"));
            printf(i18n("-version : displays the version number\n"));
            printf(i18n("-num # : opens configuration for buttons #\n"));
            printf(i18n("And all KApplication and Qt-toolkit arguments.\n"));
            exit(0);
        }
    }
    mainKGS = new KGoodStuff(0,0,num);
    mainKGS->connect(&myapp, SIGNAL(windowAdd(Window)), 
               SLOT(windowAdd(Window)));
#if 0
    mainKGS->connect(&myapp, SIGNAL(init()), 
               SLOT(kwmInit()));
    mainKGS->connect(&myapp, SIGNAL(windowRemove(Window)), 
               SLOT(windowRemove(Window)));
    mainKGS->connect(&myapp, SIGNAL(windowChange(Window)), 
               SLOT(windowChange(Window)));
    mainKGS->connect(&myapp, SIGNAL(windowActivate(Window)), 
               SLOT(windowActivate(Window)));
    mainKGS->connect(&myapp, SIGNAL(windowIconChanged(Window)), 
               SLOT(windowIconChanged(Window)));
    mainKGS->connect(&myapp, SIGNAL(windowRaise(Window)), 
               SLOT(windowRaise(Window)));
    mainKGS->connect(&myapp, SIGNAL(desktopChange(int)), 
               SLOT(kwmDesktopChange(int)));
    mainKGS->connect(&myapp, SIGNAL(desktopNameChange(int, QString)), 
               SLOT(kwmDesktopNameChange(int, QString)));
    mainKGS->connect(&myapp, SIGNAL(desktopNumberChange(int)), 
               SLOT(kwmDesktopNumberChange(int)));
    mainKGS->connect(&myapp, SIGNAL(commandReceived(QString)), 
               SLOT(kwmCommandReceived(QString)));
    mainKGS->connect(&myapp, SIGNAL(dockWindowAdd(Window)), 
               SLOT(dockWindowAdd(Window)));
    mainKGS->connect(&myapp, SIGNAL(dockWindowRemove(Window)), 
               SLOT(dockWindowRemove(Window)));
//   mainKGS->connect(&myapp, SIGNAL(playSound(QString)), 
// 		     SLOT(playSound(QString)));
    mainKGS->connect(&myapp, SIGNAL(kdisplayPaletteChanged()), 
               SLOT(kdisplayPaletteChanged()));
#endif
    myapp.setMainWidget(mainKGS);
    // connect to kwm as docking module
    myapp.connectToKWM(true);
//    mainKGS->launchSwallowedApplications();
    mainKGS->show();
    myapp.syncX();
    myapp.processEvents();
    XSelectInput(qt_xdisplay(), qt_xrootwin(), KeyPressMask);
    kapp->getConfig()->setGroup("kgoodstuff");
//    grabKey(XK_F1, Mod1Mask);
    while (1)
        myapp.exec(); 
    return 0;
}

bool isKdelnkFile(const char* filename)
{
    QFile file(filename);
    if (file.open(IO_ReadOnly)){
        char s[64];
        int r = file.readBlock(s, 64);
        file.close();
        
        if(r == -1)
            return false;
        
        // terminate string
        s[r] = '\0';
        return (QString(s).find("[KDE Desktop Entry]", 0, false) != -1);
    }
    return false;
}
